(:~
 : This modules handles everything concerning the interactive view on the BdN
 : website. Its main function, digital:transform, is triggered by xml-utils:make-html-digital-view.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 :)

xquery version "3.1";

module namespace digital="http://bdn-edition.de/ns/digital";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

(: import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "utils.xqm"; :)
import module namespace commons="http://bdn-edition.de/ns/commons" at "commons.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace single="http://bdn-edition.de/ns/single" at "single.xqm";

(: for color palette cf. http://colorpalettes.net/color-palette-3653/  and
 : http://colorpalettes.net/color-palette-1481/ plus custom colors :)
declare variable $digital:colors :=
  map {
    "a": "#359d93",
    "ab":  "#533380",
    "b":  "#823066",
    "c":  "#df6b6a",
    "d":  "#f4b771",
    "e":  "#abd1e8",
    "f":  "#43640b",
    "z":  "#afca54",
    "bc": "#ded96f",
    "bd": "#3e5e04",
    "other": "#a4bfd1",
    "quote": "#79a6d2"
  };

(:~
 : The main transformation routine. It iterates all nodes of a given TEI/XML
 : file in the BdN specific intermediate format and procudes the HTML output for
 : the digital view.
 :
 : This function is called by xml-utils:make-html-digital-view.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @version 1.0
 : @param $nodes the nodes of a given TEI/XML document
 : @param $filename e.g. "2228v.xml"
 : @return the respective HTML nodes
 :)
declare function digital:transform($nodes as node()*, $filename as xs:string)
as node()*
{
  for $node in $nodes
  return
  ( if ($node/@xml:id) then
      commons:make-text-anchor(string($node/@xml:id))
    else
      (),
    try
    { typeswitch($node)
      case text() return
        text{replace($node, "@", " ")}

      case comment() return
        ()

      case element(tei:teiHeader) return
        ()

      case element(tei:text) return
        <div class="col-md-11">
          {digital:transform($node/node(), $filename)}
        </div>

      case element(tei:front) return
        element xhtml:div
        {
          attribute class {"front"},
          digital:transform($node/node(), $filename)
        }


      case element(tei:titlePage) return
        element xhtml:span
        {
          attribute class {"title-page"},
          digital:transform($node/node(), $filename)
        }

      case element(tei:titlePart) return
        if ($node/@type = "main") then
          if ($node/ancestor::tei:rdg[@type = ("ppl", "ptl")]) then
            element xhtml:span
            {
              attribute class {"rdg-head"},
              digital:transform($node/node(), $filename)
            }
          else
            element xhtml:h1
            {
              attribute class {"main-title"},
              digital:transform($node/node(), $filename)
            }
        else if ($node/@type = "sub") then
          if ($node/ancestor::tei:rdg[@type = ("v", "pt", "pp")]) then
            element xhtml:span
            {
              attribute class {"rdg-head"},
              digital:transform($node/node(), $filename)
            }
          else
            element xhtml:h2
            {
              attribute class {"sub-title"},
              digital:transform($node/node(), $filename)
            }
        else if ($node/@type = "volume") then
          element xhtml:span
          {
            attribute class {"volume"},
            digital:transform($node/node(), $filename)
          }
        else
          ()

      case element(tei:div) return
        if ($node[@type = ("section", "contents", "preface", "corrigenda")]) then
          element xhtml:div
          {
            attribute class {$node/@type},
            digital:transform($node/node(), $filename)
          }
        else if ($node[@type = "synopsis"]) then
          (digital:transform($node/tei:head, $filename),
          <table>
            <tr>
              <td>
                {digital:transform($node/tei:div[@type = 'letter'][1]/tei:head, $filename)}
              </td>
              <td>
                {digital:transform($node/tei:div[@type = 'letter'][2]/tei:head, $filename)}
              </td>
            </tr>
            {   (: ATTENTION: this implies that the two div[@type = 'letter'] always have the same number of ab :)
              for $ab in $node/descendant::tei:div[@type = 'letter'][1]/tei:ab
                let $corresp :=
                  $node/tei:div[@type = 'letter'][2]/tei:ab[@xml:id = $ab/@corresp]
                return
                  <tr>
                    <td class="syn-td">
                      {digital:transform($ab, $filename)}
                    </td>
                    <td class="syn-td">
                      {digital:transform($corresp, $filename)}
                    </td>
                  </tr>
            }
          </table>)
        else if ($node/@type = "letter" and $node/ancestor::tei:div[@type = "synopsis"]) then
          ()
        else
          element xhtml:div
          {
            digital:transform($node/node(), $filename)
          }

      case element(tei:head) return
        switch($node/@type)
        case "main" return
          element xhtml:div
          {
            attribute class {"list-main-head"},
            digital:transform($node/node(), $filename)
          }
        case "sub" return
          element xhtml:div
          {
            attribute class {"list-sub-head"},
            digital:transform($node/node(), $filename)
          }
        default return
          element xhtml:h2
          {
            digital:transform($node/node(), $filename)
          }

      case element(tei:p) return
          element xhtml:div
          {
              if ($node/@rend= "center-aligned") then
                  attribute class {"center-aligned"}
              else
                  attribute class {"p"},
              digital:transform($node/node(), $filename)
          }

      case element(tei:seg) return
          if ($node/@type = "margin") then
              element xhtml:span
              {
                  attribute class {"margin"},
                  text{"{"},
                  digital:transform($node/node(), $filename),
                  text{"}"}
              }
          else
              digital:transform($node/node(), $filename)

      case element(tei:list) return
        element xhtml:ul
        {
          (if ($node/parent::tei:item) then
            attribute class {"nested-list"}

          else if ($node/ancestor::tei:rdg) then
            attribute class {"rdg-list"}

          else
            attribute class {"text-list"}),
            digital:transform($node/node(), $filename)
        }

      case element(tei:item) return
        element xhtml:li
        {
          (if ($node/ancestor::tei:div[@type = "contents"]) then
            attribute class {"list-item-toc"}
          else
            attribute class {"list-item"}),
          digital:transform($node/node(), $filename)
        }

      case element(tei:milestone) return
        let $edition := replace(replace($node/@edRef, "#", ""), " ", "")
        return
        if ($node/@unit = "fn-break") then
          element xhtml:span
          {
            attribute class {"fn-break"},
            string("|" || $edition || $node/@n || "|")
          }
        else
          let $edition := replace($node/@edRef, "[# ]+", "")
          let $ancestor-rdg-type := $node/ancestor::tei:rdg[1]/@type
          return
            if ($ancestor-rdg-type = ("ppl", "ptl")
            and ($node/preceding-sibling::*[1][not(self::tei:pb
              or self::tei:list
              or self::tei:rdgMarker)]
              or $node/parent::tei:seg/preceding-sibling::node())) then
              (<br />,
              if ($node/@unit = "p") then
                <span class="filler" />
              else
                ())
            else
              element xhtml:span
              {
                attribute class {"milestone"},
                switch($node/@unit)
                  case "p" return "⌇⌇"
                  case "no-p" return <s>⌇⌇</s>
                  case "line" return "⌇"
                  case "no-line" return <s>⌇</s>
                  default return "",
                <sup>{$edition}</sup>,
                " "
              }

      case element(tei:pb) return
        element xhtml:span
        {
          attribute{"class"}{"pb"},
          concat
          (
            string("|" || replace($node/@edRef, "[# ]+", "")),
            (if ($node/@type = "sp") then string("[") else ""),
            string($node/@n),
            (if ($node/@type = "sp") then string("]") else ""),
            string("|")
          )
          }

      case element(tei:cb) return
        element xhtml:span
        {
          attribute{"class"}{"cb"},
          concat
          (
            string("|" || replace($node/@edRef, "[# ]+", "")),
            string($node/@n),
            string("|")
          )
        }

      case element(tei:supplied) return
        (
          text {"["},
          digital:transform($node/node(), $filename),
          text {"]"}
        )


      case element(tei:line) return
        element xhtml:span
        {
          attribute class {"line"},
          digital:transform($node/node(), $filename)
        }

      case element(tei:lg) return
        element xhtml:div
        {
          attribute class {"linegroup"},
          digital:transform($node/node(), $filename)
        }

      case element(tei:row) return
        if ($node/preceding-sibling::tei:row) then
          <table><tr>
            {digital:transform($node/node(), $filename)}
          </tr></table>
        else
          <tr>{digital:transform($node/node(), $filename)}</tr>

      case element(tei:cell) return
        <td>
          {
          if ($node/@role = "label") then
            <b>{digital:transform($node/node(), $filename)}</b>
          else
            digital:transform($node/node(), $filename)
          }
        </td>

      case element(tei:lb) return
        if ($node/preceding-sibling::*[1][self::tei:titlePart
          or self::tei:head]) then
          ()
        else
          <br />

      case element(tei:index) return
        ()

      case element(tei:hi) return
        if ($node/@rend) then
          element xhtml:span
          {
            if ($node/@rend = "spaced-out"
            and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
                attribute class {"spaced-out spaced-out-no-right-padding"}
            else
                attribute class {$node/@rend},
            digital:transform($node/node(), $filename)
          }
        else
          <em>{digital:transform($node/node(), $filename)}</em>

      case element(tei:aligned) return
        element xhtml:span
        {
          attribute class {$node/@rend},
          digital:transform($node/node(), $filename)
        }

      case element(tei:foreign) return
        if ($node/@xml:lang ="grc") then
          element xhtml:span
          {
            attribute class {"greek"},
            digital:transform($node/node(), $filename)
          }
        else
          digital:transform($node/node(), $filename)

      case element(tei:label) return
        if ($node/@type = "headword") then
          <b>{digital:transform($node/node(), $filename)}</b>
        else
          element xhtml:span
          {
            attribute class {"label-wrapper"},
            digital:transform($node/node(), $filename)
          }

      case element(tei:ab) return
        if ($node[@type = "half-title"]) then
          element xhtml:div
          {
            attribute class {$node/@type},
            digital:transform($node/node(), $filename)
          }
        else
          digital:transform($node/node(), $filename)

      case element(tei:choice) return
        if ($node/tei:abbr) then
          digital:transform($node/tei:abbr, $filename)

        else if ($node/tei:orig) then
          digital:transform($node/tei:orig, $filename)

        else if ($node/tei:corr[@type = "editorial"]) then
          digital:transform($node/tei:corr[@type = "editorial"], $filename)

        else if ($node/tei:corr[@type = "authorial"]) then
          (
            digital:transform($node/tei:corr, $filename),
            text{" "},
            element xhtml:a
            {
              attribute href {commons:get-link-url($node, "digital")},
              <i class="fas fa-pen-fancy"></i>
            }
          )
        else
          ()

      case element(tei:abbr) return
        (: for generating the id $node/node() isn't sufficient because some
        abbreviations are so common that they appear several times per chapter
        which results in 1 < panels to open :)
        let $hash-input := serialize($node) || util:random()
        let $id := util:hash(generate-id($node), "md5")
        let $ref := "#" || $id
        return
          (
              element xhtml:span
              {
                attribute class {"expan-content"},
                attribute href {$ref},
                attribute data-toggle {"modal"},
                digital:transform($node/node(), $filename),
                digital:make-modal($node, $id, $filename)
              }
          )

      case element(tei:persName) return
        (
          digital:transform($node/node(), $filename),
          if ($node/@ref) then
            let $id := util:hash(generate-id($node/node()[1]), "md5")
            let $ref := "#" || $id
            let $uri := replace($node/@ref, "^#", "")
            return
              (
                text{" "},
                element xhtml:span
                {
                  attribute class {"fa-user-link"},
                  attribute id {$id},
                  attribute data-target {$ref},
                  attribute tgURI {$uri},
                  <i class="fas fa-user"></i>
                }
              )
          else
            ()
        )

      case element(tei:anchor) return
        commons:make-text-anchor($node/@xml:id)

      case element(tei:ptr) return
        let $id :=
          if ($node/@id) then
            util:hash($node/@id, "md5")
          else
              util:hash(generate-id($node), "md5")
        let $ref := "#" || $id

        return
          (
              text{" "},
            if ($node/@type = "editorial-commentary") then
              (
                digital:transform($node/node(), $filename),
                element xhtml:span
                {
                  attribute class {$node/@type},
                  attribute href {$node/@target},
                  attribute data-toggle {"modal"},
                  <i class="far fa-comment"></i>
                }
              )
            (: else encompasses $node/@type = "bibliographic-object" :)
            else
              element xhtml:span
              {
                attribute class {"fa-book-link"},
                attribute id {$id},
                attribute data-target {$ref},
                attribute tgURI {$node/@target},
                <i class="fas fa-book"></i>
              },
            digital:make-modal($node, $id, $filename)
          )

      case element(tei:ref) return
        (
          element xhtml:span
          {
            attribute class {"ref"},
            digital:transform($node/node(), $filename),
            string(" "),
            element xhtml:a
            {
              attribute class {"text-link"},
              attribute href {commons:get-link-url($node, "digital")},
              <i class="fas fa-external-link-alt"></i>
            }
          },
        if (commons:starts-next-text-with-punctuation($node, "digital")) then
          ()
        else
          text{" "}
        )

      case element(tei:app) return
        if ($node/@type = "structural-variance") then
          digital:transform($node/tei:lem, $filename)
        else
          let $textcritical-rdgs := $node/child::*[@type = ("om", "v", "pp", "pt", "ppl", "ptl")]
          let $textcritical-no := count($textcritical-rdgs)

          let $typo-corrs := $node/child::*[@type = ("typo-correction")]
          return
            (
              if ($typo-corrs) then
                digital:make-typo-corrs($node, $filename)
              else
                digital:transform($node/tei:lem, $filename),
              for $rdg in $textcritical-rdgs return
                (
                  digital:make-rdg($rdg, $filename),
                  if ($textcritical-no gt 1
                  and $rdg = $textcritical-rdgs[last()]) then
                    ()
                  else
                    <div class="rdg-digital-divider" />
                )
            )

      case element(tei:lem) return
          let $relevant-siblings := $node/following-sibling::*[@type = ("om", "v", "pp", "pt", "ppl", "ptl")]
          let $relevant-wits := string-join($relevant-siblings/@wit, "")
          let $relevant-wits := replace($relevant-wits, "[# ]+", "")
          let $id := util:hash(generate-id($node/parent::*), "md5")
          return
              if ($node/@type = "missing-structure") then
                  ()
              else if ($node/following-sibling::*[@type = ("pt", "ptl")]) then
                  let $insertion-rdgs := $node/following-sibling::*[@type = ("pt", "ptl")]
                  let $insertion-wits := string-join($insertion-rdgs/@wit, "")
                  let $insertion-wits := replace($insertion-wits, "[# ]+", "")
                  return
                      element xhtml:span
                      {
                          attribute class {"lem insertion"},
                          attribute id {$id},
                          attribute wits {$insertion-wits},
                          <sup>{$insertion-wits}</sup>,
                          text {"√"}
                      }

              else if (count($node/*) = 1
              and $node/tei:item) then
                  element xhtml:li
                  {
                    (
                      if ($node/ancestor::tei:div[@type = "contents"]) then
                        attribute class {"list-item-toc"}
                      else
                        attribute class {"list-item"}
                    ),
                    element xhtml:span
                    {
                      attribute class
                      {
                        "lem" ||
                        (
                            if ($node/tei:div
                            or $node/tei:titlePage
                            or $node/tei:p
                            or $node/tei:note) then
                                  " lem-block"
                              else
                                  ()
                        )
                      },
                      attribute id {$id},
                        if ($relevant-wits) then
                          attribute wits {$relevant-wits}
                        else
                          (),
                        digital:transform($node/tei:item/node(), $filename)
                    }
                  }

              else
                  element xhtml:span
                  {
                      attribute class
                      {
                        "lem" ||
                        (
                          if ($node/tei:div
                          or $node/tei:titlePage
                          or $node/tei:p
                          or $node/tei:note) then
                            " lem-block"
                          else
                            ()
                        )
                      },
                      attribute id {$id},
                      if ($relevant-wits) then
                          attribute wits {$relevant-wits}
                      else
                          (),
                      digital:transform($node/node(), $filename)
                  }

      case element(tei:note) return
        if ($node/@type = "editorial-commentary") then
          digital:make-modal($node, $node/@xml:id, $filename)
        else if ($node/@type = "editorial-footnote") then
            let $id := util:hash(generate-id($node), "md5")
            let $target := "#" || $id
            return
              (
                element xhtml:span
                {
                    attribute class {$node/@type},
                    attribute href {$target},
                    attribute data-toggle {"modal"},
                    <i class="far fa-comment"></i>
                },
              digital:make-editorial-footnote($node, $id, $filename)
              )
        else
          element xhtml:div
          {
            attribute class {"note"},
            digital:transform($node/node(), $filename)
          }

      case element (tei:floatingText) return
          element xhtml:div
          {
              attribute class {"floating-text"},
              digital:transform($node/node(), $filename)
          }

      case element (tei:signed) return
          element xhtml:div
          {
              attribute class {"signed"},
              digital:transform($node/node(), $filename)
          }

      case element (tei:quote) return
          (
            element xhtml:span
            {
                attribute class {"quote"},
                attribute id {util:hash(generate-id($node), "md5")},
                attribute target-node {util:hash(generate-id(text {$node/text()}), "md5")},
                digital:transform($node/node(), $filename),
                <span class="quote-icon">
                    <i class="fas fa-quote-right"></i>
                </span>
            },
            digital:make-quote($node, $filename)
          )

      case element ( tei:opener ) return
        if ( $node/@rend = "center-aligned" ) then
          element div { 
            attribute class { "center-aligned opener" },
            digital:transform($node/node(), $filename)
          }
        else
          ()

      default return
        digital:transform($node/node(), $filename),
        commons:make-whitespace-after($node)
    } catch *
    { element xhtml:p
          {
              attribute class {"error"},
              text{"Error: "}, $err:code,
              text{"Description: "}, $err:description,
              text{"Node: "}, $node/name()
          }
    }
  )
};


(:~
 : Creates a modal dialog for an abbreviation expansion or the display of an
 : editorial commentary. Since both use cases nearly have the same requirements
 : both are handled by this function.
 :
 : This function is called by digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node a tei:abbr or a tei:note[@type = "editorial-commentary"]
 : @param $id
 : @param $filename e.g. "2228v-if.xml"
 : @return xhtml:div the modal dialog
 :)
declare function digital:make-modal($node as node(), $id as xs:string,
$filename as xs:string*) as element(xhtml:div)
{


  let $title :=
    if ($node[self::tei:abbr]) then
      let $expans :=
        for $expan in $node/following-sibling::tei:expan return
          (
            digital:transform($expan, $filename),
            if (not($expan = $node/following-sibling::tei:expan[last()])
            and count($node/following-sibling::tei:expan) gt 1) then
              ", "
            else
              ()
          )
      let $expans := string-join($expans)
      (: the abbr node can contain another entity
       : -> its text() would be empty and is contained
       : in a child node
       :)
      let $abbr :=
        if ($node[self::tei:abbr]/text()) then
          $node[self::tei:abbr]/text()
        else
          $node[self::tei:abbr]/*/text()
      let $child-nodes :=
        for $child-node in $abbr return
          digital:transform($child-node, $filename)
      let $child-nodes := string-join($child-nodes)
      return
        "Abkürzungsauflösung von """ || $child-nodes || """: " || $expans
    else if ($node/@type = "editorial-commentary") then
      digital:transform($node/tei:label, $filename)
    else ()

  let $content :=
    if ($node/@type = "editorial-commentary") then
      digital:transform($node/tei:p, $filename)
    else ()

  return
	  <div class="modal fade" role="dialog" id="{$id}">
  	    <div class="modal-dialog abbr-or-commentary-modal">
	      <div class="modal-content">
  	      <div class="modal-header">
	          <button type="button" class="close" data-dismiss="modal">
  	          × <!-- times -->
	          </button>
            <h5 class="modal-title abbr-or-commentary-head">
              {
                if ($node[self::tei:abbr]) then
                  ()
                else
                  <i class="far fa-comment"></i>
              }
              {" " || $title}
	          </h5>
	        </div>
	        {
            if (not($node[self::tei:abbr])) then
              <div class="modal-body">
                <table>
                  <tr>
                    <td>{$content}</td>
                  </tr>
                </table>
              </div>
          else
            ()
          }
        </div>
      </div>
    </div>
};


declare function digital:make-editorial-footnote($node as node(), $id as xs:string,
$filename as xs:string*) as element(xhtml:div)
{
    <div class="modal fade" role="dialog" id="{$id}">
        <div class="modal-dialog abbr-or-commentary-modal">
            <div class="modal-content">
                <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal">
                × <!-- times -->
                </button>
                <h5 class="modal-title abbr-or-commentary-head">
                    {digital:transform($node/node(), $filename)}
                </h5>
                </div>
            </div>
        </div>
    </div>
};

(:~
 : Creates an xhtml:span for a tei:rdg that is relevant for textual criticism,
 : i.e. a tei:rdg[@type = ("om", "v", "pp", "pt", "ppl", "ptl")], and an
 : xhtml:path that belongs to it.
 :
 : While the xhtml:span will be displayed in the div[@id = "variants"] on the
 : page, the xhtml:path will be part of the SVG layer. Moving the elements that
 : are created in this function to the right place is handled by xml-utils:make-html-digital-view.
 :
 : This function is called by digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node a tei:rdg
 : @param $filename e.g. "2228v-if.xml"
 : @return sequence of xhtml:span and xhtml:path
 :)
declare function digital:make-rdg($node as element(tei:rdg), $filename as xs:string)
as element()*
{
  let $wits := replace($node/@wit, "[#]+", "")
  let $color := digital:get-rdg-color($wits)

  let $wits := replace($wits, " ", "")
  let $rdg-id := util:hash(generate-id($node), "md5")
  let $lem-id := util:hash(generate-id($node/parent::*), "md5")
  let $target-lem := "#" || $lem-id
  let $target-rdg := "#" || $rdg-id
  return
  (
    element xhtml:span
    {
      attribute class {"rdg-digital rdg-" || $wits},
      attribute id {$rdg-id},
      attribute target-lem {$target-lem},
      attribute wits {$wits},
      <table>
        <tr>
          <td class="siglum">
            <strong>{$wits}:</strong>
          </td>
          <td>
            {if ($node/@type = "om") then
              <i>&#248;</i>
            else
              digital:transform($node/node(), $filename)
            }
          </td>
        </tr>
      </table>
    },
    element xhtml:path
    {
      attribute class {"rdg-path"},
      attribute d {"M10 10"},
      attribute target-rdg {$target-rdg},
      attribute rdg-wits {$wits},
      attribute target-lem {$target-lem},
      attribute stroke {$color},
      attribute fill {"transparent"}
    }
  )
};


declare function digital:make-quote($node as element(tei:quote),
$filename as xs:string) as element()*
{
    let $text-content := text {$node/text()}
    let $id := util:hash(generate-id($text-content), "md5")
    let $corresp-ids := tokenize($node/@corresp, " ")

    let $corresp-ids :=
        for $corr-id in $corresp-ids return
            substring-after($corr-id, "#")

    let $no-of-quoted-texts := count($corresp-ids)

    let $target-id := "#" || util:hash(generate-id($node), "md5")

    let $corresp-nodes :=
        for $corr-id in $corresp-ids return
            (: in this case we are in a split file and $filename contains the author's name :)
            if (contains($filename, "-")) then
                let $author :=
                    if (contains($filename, "bahrdt-semler")) then
                        "bahrdt-semler"
                    else
                        substring-before($filename, "-")
                let $full-file := doc($config:full-xmls || $author || "-full.xml")
                return
                    $full-file//*[@xml:id = $corr-id]

            (: in this case we are in a single file and $filename is the TextGrid URI :)
            else
                for $doc in collection($config:sade-data) return
                    if ($doc//*[@xml:id = $corr-id]
                    and not(matches(base-uri($doc), "full"))) then
                        $doc//*[@xml:id = $corr-id]
                    else
                        ()

    let $target-urls :=
        for $corr-node in $corresp-nodes return
            (: in this case we are in a split file and all IDs are listed in the ID file :)
            if (contains($filename, "-")) then
                commons:get-link-url($corr-node, "digital")

            (: in this case we are in a single file and ID are not listed in the ID file :)
            else
                for $doc in collection($config:sade-data) return
                    if ($doc//* = $corr-node
                    and not(matches(base-uri($doc), "full"))) then
                        commons:get-link-url-to-corresp-node($corr-node, base-uri($doc), "digital")
                    else
                        ()

    let $quoted-texts :=
        for $corr-node in $corresp-nodes return
            $corr-node/ancestor::tei:text//tei:supplied[@reason = "toc-title"][1]

    return
        (
            element xhtml:div
            {
                attribute class {"quoted-text"},
                attribute id {$id},
                attribute target-node {$target-id},
                for $iii in 1 to $no-of-quoted-texts return
                    <table class="quoted-passage" count="{$iii}">
                        <tr>
                            <td class="siglum">
                                Zitat aus <span class="quoted-work">{$quoted-texts[$iii]/string()}</span>:
                            </td>
                        </tr>
                        <tr>
                            <td class="direct-quote">
                                {
                                    digital:transform($corresp-nodes[$iii], $filename)
                                }
                            </td>
                        </tr>
                        <tr>
                            <td class="quote-link">
                                <a href="{$target-urls[$iii]}">
                                    Gehe zum zitierten Text
                                    <i class="fas fa-external-link-alt"></i>
                                </a>
                            </td>
                        </tr>
                    </table>
            },
            element xhtml:path
            {
                attribute class {"quote-path"},
                attribute d {"M10 10"},
                attribute target-quoting {$target-id},
                attribute target-quoted {"#" || $id},
                attribute stroke {map:get($digital:colors, "quote")},
                attribute fill {"transparent"}
            }
        )
};


(:~
 : Retrieves the color that belongs to one or several witnesses of a tei:rdg.
 :
 : This function is called by digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $wits tei:rdg/@wit
 : @return the hex color codes a witness
 :)
declare function digital:get-rdg-color($wits as xs:string*) as xs:string
{
    if (count(tokenize($wits, " ")) gt 1) then
        let $wit-combination :=
            tokenize($wits, " ")
            => string-join("")
        let $color-hex := map:get($digital:colors, $wit-combination)
        return
            if ($color-hex) then
                $color-hex
            else
                map:get($digital:colors, "other")
    else
      map:get($digital:colors, $wits)
};


(:~
 : Triggers the creation of a modal dialog for compositor's errors in different
 : printings and the base text which have have been corrected either by the
 : author himself or by modern editors.
 :
 : This function is called by digital:transform.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node a tei:app which has a child with a compositor's error
 : @param $file the filename, e.g. "2228v.xml"
 : @return the text to be displayed and the (hidden) modal dialog
 :)
declare function digital:make-typo-corrs($node as element(tei:app),
$file as xs:string) as node()*
{
  let $typo-corr-rdgs := $node/child::tei:rdg[@type = "typo-correction"][descendant::tei:choice]
  let $typo-corr-lem := $node/child::tei:lem[descendant::tei:corr]
  let $wit :=
    if ($typo-corr-rdgs) then
      ()
    else
      (: single file names don't have a hyphen, only the ones that are the
      result of semantic splitting :)
      if (matches($file, "-")) then
        let $author := substring-before($file, "-")
        let $full-file := doc($config:full-xmls || $author || "-full.xml")
        return $full-file//tei:witness[@n = "base-text"]/@xml:id/string()
      else
        $node/ancestor::tei:TEI//tei:witness[@n = "base-text"]/@xml:id

  return
    if ($typo-corr-rdgs) then
      for $typo in $typo-corr-rdgs return
        (digital:transform($typo//tei:choice, $file),
        single:make-typo-corr($typo//tei:choice[child::tei:corr], replace($typo/@wit, "[# ]", ""), $file))
    else
      (
        digital:transform($typo-corr-lem//tei:choice, $file),
        try {
          single:make-typo-corr($typo-corr-lem//tei:choice[child::tei:corr], $wit, $file)
        } catch * {
          util:log("debug", $err:code || $err:description || $node/name()),
          element xhtml:p
          {
              attribute class {"error"},
              text{"Error: "}, $err:code,
              text{"Description: "}, $err:description,
              text{"Node: "}, $node/name()
          }
        }
      )
};
